"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const vscode_uri_1 = require("vscode-uri");
const theme_check_common_1 = require("@shopify/theme-check-common");
const diagnostics_1 = require("../../diagnostics");
const documents_1 = require("../../documents");
const FixProvider_1 = require("./FixProvider");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
(0, vitest_1.describe)('Unit: FixProvider', () => {
    const uri = vscode_uri_1.URI.file('/path/to/file.liquid').toString();
    const contents = `
    {% assign x = 1 %}
    <script src="2.js"></script>
    <script src="3.js"></script>
  `;
    const document = vscode_languageserver_textdocument_1.TextDocument.create(uri, 'liquid', 0, contents);
    let documentManager;
    let diagnosticsManager;
    let fixProvider;
    function makeOffense(checkName, needle, unfixable) {
        const start = contents.indexOf(needle);
        const end = start + needle.length;
        return {
            type: theme_check_common_1.SourceCodeType.LiquidHtml,
            check: checkName,
            message: 'Parser blocking script detected',
            absolutePath: '/path/to/file.liquid',
            severity: theme_check_common_1.Severity.ERROR,
            start: { ...document.positionAt(start), index: start },
            end: { ...document.positionAt(end), index: end },
            fix: unfixable ? undefined : (corrector) => corrector.replace(18, 36, 'fixed'),
        };
    }
    (0, vitest_1.beforeEach)(() => {
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new diagnostics_1.DiagnosticsManager({ sendDiagnostics: vitest_1.vi.fn() });
        documentManager.open(uri, contents, 1);
        fixProvider = new FixProvider_1.FixProvider(documentManager, diagnosticsManager);
    });
    (0, vitest_1.describe)('When multiple offenses of various types exist, and the selection includes an offense with at least one more of the same type in the file', () => {
        (0, vitest_1.beforeEach)(() => {
            diagnosticsManager.set(uri, 1, [
                makeOffense('UnusedAssign', '{% assign x = 1 %}'),
                makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
                makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
            ]);
        });
        (0, vitest_1.it)('provides all code actions categories', async () => {
            const codeActions = fixProvider.codeActions({
                textDocument: { uri },
                range: {
                    start: document.positionAt(contents.indexOf('2.js')),
                    end: document.positionAt(contents.indexOf('2.js')),
                },
                context: { diagnostics: [] },
            });
            (0, vitest_1.expect)(codeActions.length).toBe(3);
            const [fixCursorAction, fixSameTypeAction, fixAllAction] = codeActions;
            (0, vitest_1.expect)(fixCursorAction).toEqual({
                title: 'Fix this ParserBlockingScript problem: Parser blocking script detected',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: true,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [1]],
                },
            });
            (0, vitest_1.expect)(fixSameTypeAction).toEqual({
                title: 'Fix all ParserBlockingScript problems',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [1, 2]],
                },
            });
            (0, vitest_1.expect)(fixAllAction).toEqual({
                title: 'Fix all auto-fixable problems',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [0, 1, 2]],
                },
            });
        });
    });
    (0, vitest_1.describe)('When multiple offenses of various types exist, and the selection includes an offense without more of the same type in the file', () => {
        (0, vitest_1.beforeEach)(() => {
            diagnosticsManager.set(uri, 1, [
                makeOffense('UnusedAssign', '{% assign x = 1 %}'),
                makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            ]);
        });
        (0, vitest_1.it)('provides code actions, and the fix all same type is omitted', async () => {
            const codeActions = fixProvider.codeActions({
                textDocument: { uri },
                range: {
                    start: document.positionAt(contents.indexOf('2.js')),
                    end: document.positionAt(contents.indexOf('2.js')),
                },
                context: { diagnostics: [] },
            });
            (0, vitest_1.expect)(codeActions.length).toBe(2);
            const [fixCursorAction, fixAllAction] = codeActions;
            (0, vitest_1.expect)(fixCursorAction).toEqual({
                title: 'Fix this ParserBlockingScript problem: Parser blocking script detected',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: true,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [1]],
                },
            });
            (0, vitest_1.expect)(fixAllAction).toEqual({
                title: 'Fix all auto-fixable problems',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [0, 1]],
                },
            });
        });
    });
    (0, vitest_1.describe)('When only one offense type exists, and the selection includes an offense', () => {
        (0, vitest_1.beforeEach)(() => {
            diagnosticsManager.set(uri, 1, [
                makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
                makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
            ]);
        });
        (0, vitest_1.it)('provides code actions, and the fix all code action is omitted', async () => {
            const codeActions = fixProvider.codeActions({
                textDocument: { uri },
                range: {
                    start: document.positionAt(contents.indexOf('2.js')),
                    end: document.positionAt(contents.indexOf('2.js')),
                },
                context: { diagnostics: [] },
            });
            (0, vitest_1.expect)(codeActions.length).toBe(2);
            const [fixCursorAction, fixSameTypeAction] = codeActions;
            (0, vitest_1.expect)(fixCursorAction).toEqual({
                title: 'Fix this ParserBlockingScript problem: Parser blocking script detected',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: true,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [0]],
                },
            });
            (0, vitest_1.expect)(fixSameTypeAction).toEqual({
                title: 'Fix all ParserBlockingScript problems',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [0, 1]],
                },
            });
        });
    });
    (0, vitest_1.describe)('When there is only one offense', () => {
        (0, vitest_1.beforeEach)(() => {
            diagnosticsManager.set(uri, 1, [
                makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            ]);
        });
        (0, vitest_1.it)('provides a code action to fix it', async () => {
            const codeActions = fixProvider.codeActions({
                textDocument: { uri },
                range: {
                    start: document.positionAt(contents.indexOf('2.js')),
                    end: document.positionAt(contents.indexOf('2.js')),
                },
                context: { diagnostics: [] },
            });
            (0, vitest_1.expect)(codeActions.length).toBe(1);
            const [fixCursorAction] = codeActions;
            (0, vitest_1.expect)(fixCursorAction).toEqual({
                title: 'Fix this ParserBlockingScript problem: Parser blocking script detected',
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: true,
                command: {
                    title: 'applyFixes',
                    command: 'themeCheck/applyFixes',
                    arguments: [uri, 1, [0]],
                },
            });
        });
    });
    (0, vitest_1.it)('provides code actions for a set of fixable offenses', async () => {
        diagnosticsManager.set(uri, 1, [
            makeOffense('UnusedAssign', '{% assign x = 1 %}'),
            makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
        ]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('2.js')),
                end: document.positionAt(contents.indexOf('2.js')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(3);
        const [fixCursorAction, fixSameTypeAction, fixAllAction] = codeActions;
        (0, vitest_1.expect)(fixCursorAction).toEqual({
            title: 'Fix this ParserBlockingScript problem: Parser blocking script detected',
            kind: 'quickfix',
            diagnostics: vitest_1.expect.any(Array),
            isPreferred: true,
            command: {
                title: 'applyFixes',
                command: 'themeCheck/applyFixes',
                arguments: [uri, 1, [1]],
            },
        });
        (0, vitest_1.expect)(fixSameTypeAction).toEqual({
            title: 'Fix all ParserBlockingScript problems',
            kind: 'quickfix',
            diagnostics: vitest_1.expect.any(Array),
            isPreferred: false,
            command: {
                title: 'applyFixes',
                command: 'themeCheck/applyFixes',
                arguments: [uri, 1, [1, 2]],
            },
        });
        (0, vitest_1.expect)(fixAllAction).toEqual({
            title: 'Fix all auto-fixable problems',
            kind: 'quickfix',
            diagnostics: vitest_1.expect.any(Array),
            isPreferred: false,
            command: {
                title: 'applyFixes',
                command: 'themeCheck/applyFixes',
                arguments: [uri, 1, [0, 1, 2]],
            },
        });
    });
    (0, vitest_1.it)('does not provide code actions for non-fixable offenses', async () => {
        const nonFixableOffense = makeOffense('ParserBlockingScript', '<script src="2.js">', true);
        diagnosticsManager.set(uri, 1, [nonFixableOffense]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('2.js')),
                end: document.positionAt(contents.indexOf('2.js')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(0);
    });
    (0, vitest_1.it)('does not provide code actions if selection range does not overlap with the error', async () => {
        diagnosticsManager.set(uri, 1, [
            makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
        ]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('assign x = 1')),
                end: document.positionAt(contents.indexOf('= 1')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(0);
    });
});
//# sourceMappingURL=FixProvider.spec.js.map