"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const vscode_uri_1 = require("vscode-uri");
const theme_check_common_1 = require("@shopify/theme-check-common");
const diagnostics_1 = require("../../diagnostics");
const documents_1 = require("../../documents");
const ApplyFixesProvider_1 = require("./ApplyFixesProvider");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const offenseToDiagnostic_1 = require("../../diagnostics/offenseToDiagnostic");
(0, vitest_1.describe)('Unit: ApplyFixesProvider', () => {
    const uri = vscode_uri_1.URI.file('/path/to/file.liquid').toString();
    const contents = `
    {% assign x = 1 %}
    <script src="2.js"></script>
    <script src="3.js"></script>
  `;
    const document = vscode_languageserver_textdocument_1.TextDocument.create(uri, 'liquid', 0, contents);
    let connection;
    let documentManager;
    let diagnosticsManager;
    let applyFixProvider;
    function makeOffense(checkName, needle, fixable = true) {
        const start = contents.indexOf(needle);
        const end = start + needle.length;
        return {
            type: theme_check_common_1.SourceCodeType.LiquidHtml,
            check: checkName,
            message: 'Offense detected',
            absolutePath: '/path/to/file.liquid',
            severity: theme_check_common_1.Severity.ERROR,
            start: { ...document.positionAt(start), index: start },
            end: { ...document.positionAt(end), index: end },
            fix: fixable ? (corrector) => corrector.replace(start, end, 'fixed') : undefined,
        };
    }
    (0, vitest_1.beforeEach)(() => {
        connection = { sendRequest: vitest_1.vi.fn(), sendDiagnostics: vitest_1.vi.fn() };
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new diagnostics_1.DiagnosticsManager(connection);
        documentManager.open(uri, contents, 1);
        const capabilities = { hasApplyEditSupport: vitest_1.vi.fn().mockReturnValue(true) };
        applyFixProvider = new ApplyFixesProvider_1.ApplyFixesProvider(documentManager, diagnosticsManager, capabilities, connection);
    });
    (0, vitest_1.describe)('execute', () => {
        (0, vitest_1.it)('applies fixes for the given offense ids', async () => {
            const offenses = [
                makeOffense('Offense1', '{% assign x = 1 %}'),
                makeOffense('Offense2', '<script src="2.js"></script>', false),
                makeOffense('Offense3', '<script src="3.js"></script>'),
            ];
            diagnosticsManager.set(uri, 1, offenses);
            await applyFixProvider.execute(uri, 1, [0, 2]);
            (0, vitest_1.expect)(connection.sendRequest).toHaveBeenCalledWith(vitest_1.expect.anything(), {
                edit: {
                    documentChanges: [
                        {
                            textDocument: vitest_1.expect.anything(),
                            edits: [
                                {
                                    newText: 'fixed',
                                    range: {
                                        start: asLspPosition(offenses[0].start),
                                        end: asLspPosition(offenses[0].end),
                                    },
                                },
                                {
                                    newText: 'fixed',
                                    range: {
                                        start: asLspPosition(offenses[2].start),
                                        end: asLspPosition(offenses[2].end),
                                    },
                                },
                            ],
                        },
                    ],
                },
            });
            // We clean up stale diagnostics when we're done
            (0, vitest_1.expect)(connection.sendDiagnostics).toHaveBeenCalledWith({
                uri: vitest_1.expect.anything(),
                version: vitest_1.expect.any(Number),
                diagnostics: [offenses[1]].map(offenseToDiagnostic_1.offenseToDiagnostic),
            });
        });
        (0, vitest_1.it)('does not apply fixes if the document or diagnostics are not available', async () => {
            const offenses = [
                makeOffense('Offense1', '{% assign x = 1 %}'),
                makeOffense('Offense2', '<script src="2.js"></script>'),
                makeOffense('Offense3', '<script src="3.js"></script>'),
            ];
            diagnosticsManager.set(uri, 1, offenses);
            documentManager.close(uri);
            await applyFixProvider.execute(uri, 1, [0, 2]);
            (0, vitest_1.expect)(connection.sendRequest).not.toHaveBeenCalled();
        });
    });
});
function omit(obj, key) {
    const clone = { ...obj };
    delete clone[key];
    return clone;
}
function asLspPosition(position) {
    return omit(position, 'index');
}
//# sourceMappingURL=ApplyFixesProvider.spec.js.map